/*
    libgpcogl - Generic Purpose Computation on OpenGL (using shaders)
    Copyright (C) 2023 Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://www.repo.hu/projects/libgpcogl
      Lead developer: http://www.repo.hu/projects/libgpcogl/contact.html
*/


#include <GL/glut.h>

#include "gli_glut.h"

void gpcogl_glut_init(int *argc, char *argv[])
{
	glutInit(argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
}

void gpcogl_glut_uninit(void)
{
}

static int switch_context_glut(gpcogl_t *ctx)
{
	glutSetWindow(ctx->ogl_ctx.l);
	return 0;
}

static void *screen_render_user_data;
static void (*screen_render_user_func)(void *user_data);

/* work around glut API bug of no user data */
void display_for_udata(void)
{
	screen_render_user_func(screen_render_user_data);
}

static void screen_render_glut(gpcogl_t *ctx, void (*display)(void *user_data), void *user_data)
{
	screen_render_user_data = user_data;
	screen_render_user_func = display;
	glutDisplayFunc(display_for_udata);
	glutMainLoop();
	screen_render_user_data = NULL;
	screen_render_user_func = NULL;
}

static void flush_glut(gpcogl_t *ctx)
{
	glutSwapBuffers();
}

static int make_context_common(gpcogl_t *ctx)
{
	ctx->make_current = switch_context_glut;
	ctx->screen_render = screen_render_glut;
	ctx->flush = flush_glut;
	return gpcogl_init_context(ctx);
}

int gpcogl_glut_make_context_window(gpcogl_t *ctx, int wsx, int wsy)
{
	glutInitWindowSize(wsx, wsy);
	glutInitWindowPosition(100, 100);
	ctx->ogl_ctx.l = glutCreateWindow("libgpcogl debug render");
	return make_context_common(ctx);
}

int gpcogl_glut_make_context(gpcogl_t *ctx, const char *option)
{
	int res;
	glutInitWindowSize(1, 1);
	ctx->ogl_ctx.l = glutCreateWindow("libgpcogl off-screen render");
	res = make_context_common(ctx);
	gpcogl_redir_render_to_fb(ctx);
	glutHideWindow();
	return res;
}

void gpcogl_glut_destroy_context(gpcogl_t *ctx)
{
	gpcogl_uninit_context(ctx);
	glutDestroyWindow(ctx->ogl_ctx.l);
	ctx->ogl_ctx.l = -1;
}

#ifndef MULTIPLE_GLIS

#include "gli.h"

void gpcogl_gli_init(int *argc, char *argv[])
{
	gpcogl_glut_init(argc, argv);
}

void gpcogl_gli_uninit(void)
{
	gpcogl_glut_uninit();
}

int gpcogl_gli_make_context(gpcogl_t *ctx, const char *option)
{
	return gpcogl_glut_make_context(ctx, option);
}

int gpcogl_gli_make_context_window(gpcogl_t *ctx, int wsx, int wsy)
{
	return gpcogl_glut_make_context_window(ctx, wsx, wsy);
}

void gpcogl_gli_destroy_context(gpcogl_t *ctx)
{
	gpcogl_glut_destroy_context(ctx);
}
#endif
