/*
    libnoxgl - initialize OpenGL without X11 (mainly for off-screen rendering)
    Copyright (C) 2023 Tibor 'Igor2' Palinkas
    Credits: Concepts based on kmscube

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://www.repo.hu/projects/libnoxgl
      Lead developer: http://www.repo.hu/projects/libnoxgl/contact.html
*/

/* API version of the library: major.minor.patch */
extern const int noxgl_ver_major, noxgl_ver_minor, noxgl_ver_patch;

/* opaque lib context struct */
typedef struct noxgl_s noxgl_t;

/* Create a new lib context opening a device (e.g. "/dev/dri/card0");
   Return NULL on error. */
noxgl_t *noxgl_init(const char *device, int off_screen);

/* Unint a lib context and free all memory used */
void noxgl_uninit(noxgl_t *noxgl);

/* Create a new graphical context, identified by ctx:surface, which are
   returned in the provided pointers. The new graphical context is
   automatically activated. Return 0 on success. */
int noxgl_create_context(noxgl_t *noxgl, void **ctx_out, void **surface_out);

/* Destroy noxgl and free it; ctx and surface are invalidated */
void noxgl_destroy_context(noxgl_t *noxgl, void *ctx, void *surface);

/* Activate a previously created graphical context addressed as ctx:surface */
void noxgl_make_current_context(noxgl_t *noxgl, void *ctx, void *surface);

/* Print human readable information about all underlaying layers to stdout */
void noxgl_print_info(noxgl_t *noxgl);

